<?php

/*
 *  Simple shortcode which works for logged in users. It gives them link like http://site.com/?fvplayer_download=http://private-videos.site.com/video.mp4 which checks if the user is logged in and adds the signature/Secure Token and initiates the download. Also logs into Simple History plugin
 */

if( !class_exists('FV_Player_Pro_Download') ) :

class FV_Player_Pro_Download {

  function __construct() {
    add_action( 'admin_init', array( $this, 'register_meta_boxes' ), 20 );

    add_shortcode( 'fvplayer_download', array( $this, 'shortcode' ) );

    if( isset($_GET['fvplayer_download']) ) {
      add_action( 'init', array( $this, 'download') );
    }
  }

  function download() {
    if( !isset($_GET['fvplayer_download']) ) return;

    $url = $_GET['fvplayer_download'];

    $message = $url." from '".get_the_title($_GET['post_id'])."' (".$_GET['post_id'].") IP ".FV_Player_Pro()->get_client_ip();
    
    if( empty($_GET['signature']) || empty($_GET['expire']) ) {
      if( function_exists("SimpleLogger") ) SimpleLogger()->warning("Unathorized video download attempt (no signature) ".$message);
      wp_die('Error processing download.');
    }

    if( !$this->is_logged_in() ) {
      if( function_exists("SimpleLogger") ) SimpleLogger()->warning("Unathorized video download attempt (user not logged in) ".$message);
      wp_die('Error processing download.');
    }
    
    // We create different encoding versions of the URL
    $url_parsed = parse_url($url);
    $path = $url_parsed['path'];
    $path_parts = explode( '/', $path);
    $path_encoded = implode( '/', array_map( 'urlencode', $path_parts ) );
    $path_encoded_20percent = implode( '/', array_map( 'rawurlencode', $path_parts ) );

    $passed = false;
    foreach( array(
      $url,
      str_replace( ' ', '%20', $url ),
      str_replace( ' ', '+', $url ),
      str_replace( $path, $path_encoded, $url ),
      str_replace( $path, $path_encoded_20percent, $url )
    ) AS $url_check ) {
      $check = md5( $url_check.FV_Player_Pro()->get_client_ip().$_GET['expire'].NONCE_SALT.get_current_user_id());
      if( $check == $_GET['signature'] ) {
        $passed = true;
      }
    }

    if( !$passed ) {
      if( function_exists("SimpleLogger") ) SimpleLogger()->warning("Unathorized video download attempt (bad signature) ".$message);
      wp_die('Error processing download.');
    }
    
    if( $_GET['expire'] < time() ) {
      if( function_exists("SimpleLogger") ) SimpleLogger()->warning("Expired video link ".$message);
      wp_die('Your link has expired, please go back and reload the page.');
    }

    if( function_exists("SimpleLogger") ) SimpleLogger()->info("User downloaded video ".$message);

    $src = esc_url_raw($_GET['fvplayer_download']);

    // bunny stream
    global $FV_Player_Pro_Bunny_Stream;
    if( $FV_Player_Pro_Bunny_Stream->is_bunny_stream($src)) {
      // change src, example:
      // from - https://vz-7cc93c10-24a.b-cdn.net/e6e0f0a5-3aa6-48f0-9bea-dc2abe9362b4/playlist.m3u8
      // to - https://vz-7cc93c10-24a.b-cdn.net/e6e0f0a5-3aa6-48f0-9bea-dc2abe9362b4/original 
      $src = dirname($src) . '/original'; 
    }

    global $fv_fp;
    $src = $fv_fp->get_video_src($src, array( 'dynamic' => true, 'url_only' => true ) );

    // vimeo
    if( FV_Player_Pro_Vimeo()->is_vimeo( $src ) ) { 
      $objVideo = FV_Player_Pro_Vimeo()->get_vimeo( $src );
      $response = $objVideo->request->files->progressive;

      $max = 0;
      foreach( $response as $item ) {
        if( $item->height > $max ) {
          $max = $item->height;
          $src = $item->url;
        }
      }
    }

    wp_redirect($src);
    exit;
  }

  function get_link_template() {
    global $fv_fp;
    $html = '<a download="%name%" class="fvplayer_download %class%" href="%src%">%caption%</a>';
    if( isset($fv_fp->conf['pro']['download_template']) && $fv_fp->conf['pro']['download_template'] ) $html = stripslashes($fv_fp->conf['pro']['download_template']);
    return $html;
  }

  function is_logged_in() {
    if( class_exists('am4PluginsManager') && method_exists('am4PluginsManager','getAPI') ) {
      if( am4PluginsManager::getAPI() && am4PluginsManager::getAPI()->isLoggedIn() ) {
        return true;
      }
    }

    return  get_current_user_id() > 0;
  }

  function options() {
    global $fv_fp;
    ?>
    <p><?php _e('Use the <code>[fvplayer_download src="..." caption="..."]</code> shortcode to provide downloadable video to your site members - logged in users.', 'fv-player-pro'); ?></p>
    <table class="form-table2" style="margin: 5px; ">
      <tr>
        <td style="vertical-align:top"><label for="pro[download_template]"><?php _e('Link template', 'fv-player-pro'); ?>:</label></td>
        <td>
          <input type="text" size="40" name="pro[download_template]" id="pro[download_template]" value="<?php echo esc_attr($this->get_link_template()); ?>" />
          <p class="description"><?php _e('Available tags: <code>%name%</code> <code>%class%</code> <code>%src%</code> <code>%capton%</code>', 'fv-player-pro'); ?></p>
        </td>
      </tr>
      <?php $fv_fp->_get_checkbox(__('Disable right click note', 'fv-player-pro'), array('pro', 'download_no_right_click'), __('If your video server forces video download you can disable the \'Please click again with right mouse button and select "Save Link As...", or "Download Linked File".\' popup note here.', 'fv-player-pro') ); ?>
      <tr>
        <td colspan="4">
          <input type="submit" name="fv-wp-flowplayer-submit" class="button-primary" value="<?php _e('Save All Changes', 'fv-player-pro'); ?>" style="margin-top: 2ex;"/>
          <a class="button fv-help-link" href="https://foliovision.com/player/features/sharing/video-downloading-with-simple-history" target="_blank">Help</a>
        </td>
      </tr>
    </table>
    <?php
  }

  function register_meta_boxes() {
    add_meta_box( 'fv_player_pro_download', __('Download', 'fv-player-pro'), array( $this, 'options' ), 'fv_flowplayer_settings', 'normal', 'low' );
  }

  function generate_download( $args ) {
    global $post, $fv_fp;
    
    $url = trim($args['src']);

    if($args['caption'] === '') {
      $args['caption'] = 'Download';
    }

    $expire = time()+3600;
    $src = add_query_arg( 'fvplayer_download', $url, get_home_url() );
    $src = add_query_arg( 'post_id', $post->ID, $src );
    $src = add_query_arg( 'expire', $expire, $src );
    $src = add_query_arg( 'signature', md5($url.FV_Player_Pro()->get_client_ip().$expire.NONCE_SALT.get_current_user_id()), $src );
    $aSrc = explode( '/', strrev( $url ), 2 );
    $name = strrev( $aSrc[0] );

    $html = $this->get_link_template();

    $url_home = parse_url( get_home_url() );
    $url_src = parse_url( $url );

    // we need to detect if the link is external. If it's from the same domain as the website Chrome will allow download instantly.
    if( !$fv_fp->_get_option( array('pro', 'download_no_right_click') ) && !empty($url_home['host']) && !empty($url_src['host']) && $url_home['host'] != $url_src['host'] ) {
      $src = add_query_arg( 'fvplayer_download_external', true, $src );
    }

    $html = str_replace( '%name%', esc_attr($name), $html );
    $html = str_replace( '%class%', $args['class'], $html );
    $html = str_replace( '%src%', esc_attr($src), $html );
    $html = str_replace( '%caption%', $args['caption'], $html );

    return $html;
  }

  function maybe_process_playlist( $args ) {
    // check if vimeo channel/showcase
    global $FV_Player_Pro_Vimeo_Channel;
    $data = $FV_Player_Pro_Vimeo_Channel->vimeo_channel( $args );
    $output = array();

    // handle vimeo channel/showcase
    if( isset( $data['playlist'] ) ) {
      $captions = $data['caption'];

      // parse captions if not in args
      if( !empty( $captions ) ) {
        $replace_from = array('&amp;quot;','&amp;','\;','&quot;');
        $replace_to = array('"','<!--amp-->','<!--semicolon-->','"');
        $captions = str_replace( $replace_from, $replace_to, $captions );
        $captions = explode( ';', $captions );

        if( isset($captions) && count($captions) > 0 ) {
          foreach( $captions AS $key => $item ) {
            $captions[$key] = str_replace('<!--amp-->','&',$item);
          }
        }

        $current_caption =  isset($captions[0]) ? $captions[0] : '';
      } else {
        $current_caption = $args['caption'];
      }

      // first item
      $output[] = $this->generate_download( array(  'src' => $data['src'], 'class' => $args['class'], 'caption' => $current_caption ) );

      $playlist = explode( ';', $data['playlist'] );

      // playlist items
      foreach( $playlist as $index => $src ) {
        $src = explode( ',', $src );
        $current_caption = isset( $captions[ $index + 1 ] ) ? $captions[ $index + 1 ] : $args['caption'];
        $output[] = $this->generate_download( array( 'src' => $src[0], 'class' => $args['class'], 'caption' => $current_caption ) );
      }

    } else { // no playlist, just single video
      $output[] = $this->generate_download($args);
    }

    return $output;
  }

  function download_button( $output ) {
    if( count($output) > 1 ) {
      $html = '<ul class="fv_player_downloads"><li>'.implode('</li><li>', $output ).'</li></ul>';
    } else {
      $html = implode('',$output);
    }

    return $html;
  }

  function shortcode( $args ) {
    if( !$this->is_logged_in() ) return;

    global $fv_fp_scripts;
    if( !isset($fv_fp_scripts) ) $fv_fp_scripts = array(); //  todo: some better way of signaling to FV Player it should load its JS!

    $args = wp_parse_args( $args, array('caption' => __('Download'), 'class' => '' ) );
    $html = '';

    if( isset($args['id']) ) {
      global $FV_Player_Db;
      $objPlayer = new FV_Player_Db_Player( $args['id'], array(), $FV_Player_Db );
      $items = array();

      if( $objPlayer ) {
        $aVideos = $objPlayer->getVideos();
        if( count($aVideos) > 0 ) {
          foreach( $aVideos AS $k => $objVideo ) {
            $args['src'] = $objVideo->getSrc();
            $args['caption'] = $objVideo->getCaption() ? $objVideo->getCaption() : $args['caption'];
            $output = $this->maybe_process_playlist($args);

            // merge processed playlist items to main items download array
            $items = array_merge( $items, $output);
          }
        }

        $html = $this->download_button( $items );
      }
    } else if( isset($args['src']) ) {
      // shortcode can contain playlist argument
      $output = $this->maybe_process_playlist($args);

      $html = $this->download_button($output);
    }

    $html .= '<div style="display: none" class="fvplayer_download_hint">'.__('Please click again with right mouse button and select "Save Link As...", or "Download Linked File".', 'fv-player-pro').'</div>';

    return $html;
  }

}

global $FV_Player_Pro_Download;
$FV_Player_Pro_Download = new FV_Player_Pro_Download;

endif;
