<?php

if( !class_exists('FV_Player_Pro_Odysee') ) :

class FV_Player_Pro_Odysee extends FV_Player_Pro_Ajax_Loader {

  function __construct() {
    $this->aDomains      = array( 'https://odysee.com/' );

    $this->aSecureTokens = array( 'override' );

    add_filter( 'fv_flowplayer_get_mime_type', array( $this, 'set_file_type' ), 10 , 2 );

    add_filter('fv_player_meta_data', array( $this, 'fetch_odysee_data' ), 10, 2); // splash, caption

    add_filter( 'fv_player_video_checker_skip', array( $this, 'skip_video_checker'), 10, 2 ); // takes too long to load page if not skipped

    parent::__construct( array( 'key' => 'odysee', 'title' => 'Odysee') );
  }

  function args($args) {
    $args[] = 'verify';
    return $args;
  }

  function secure_link( $url, $securityKey, $ttl = false ) {
    $video_id = $this->get_video_id($url);
    $new_cache = false;

    if( !$video_id ) {
      return $url;
    }

    if ( $cached_url = $this->load_cache( $video_id ) ) {
      return $cached_url;
    }

    // we need to decode url otherwise request will fail
    $url = urldecode( $url );

    if( $this->is_live ) {
      $live_data = $this->get_live_data($url);

      if( is_array($live_data ) ) {
        if($live_data['Live']) {
          return $this->store_cache( $video_id, $live_data['VideoURL'] );
        }
      }
    }

    $api_url = str_replace( 'https://odysee.com/', 'lbry://', $url );

    $data = array(
      'jsonrpc' => '2.0',
      'method' => 'get',
      'params' => array(
        'uri' => $api_url,
        'save_file' => false
      )
    );

    $response = wp_remote_post( 'https://api.na-backend.odysee.com/api/v1/proxy?m=get', array(
      'body'    => json_encode($data),
      'headers' => array(
        'Content-Type' => 'application/json-rpc'
      ),
    ));

    if( !is_wp_error($response) ) {
      $body = wp_remote_retrieve_body( $response );

      $body = json_decode( $body, true );

      if( isset( $body['result']['streaming_url'] ) ) {
        $new_cache = $body['result']['streaming_url'];

        // Check if opening the URL gives you a HLS stream, it uses redirection
        // This can sometimes get a big MP4 file, so we only check a bit of it
        $response = wp_remote_get( $new_cache, array(
          'headers' => array(
            'Range' => 'bytes=0-65536'
          )
        ) );

        if( !is_wp_error($response) && !empty($response['http_response']) && method_exists( $response['http_response'], 'get_response_object' ) ) {
          $obj = $response['http_response']->get_response_object();

          if( !empty($obj->url) && stripos($obj->url,'.m3u8') !== false ) {
            $new_cache = $obj->url;
          }
        }

      } else {
        $_POST['error'] = 'Unable to parse the Odysee video.';
      }
    }

    return $this->store_cache( $video_id, $new_cache );
  }

  function set_file_type( $type ) {
    $args = func_get_args();
    if( isset($args[1]) ) {
       if( $this->get_video_id($args[1]) ) {
        $type = "video/fv-mp4";
      }
    }

    return $type;
  }

  function fetch_odysee_data($url, $post_id = false) {
    if( !$this->get_video_id($url) ) {
      return $url;
    }

    $response = wp_remote_get( $url );
    $videoData = false;

    if( !is_wp_error($response) ) {
      $body = wp_remote_retrieve_body( $response );

      preg_match('~<meta property="og:video:duration" content="(.*?)"/>~', $body, $duration); // match duration in meta tag
      preg_match('~<meta property="og:image" content="(.*?)"/>~', $body, $splash); // match splash in meta, maybe add #.jpg to link ?
      preg_match('~<title>(.*?)</title>~', $body, $caption); // match caption in title

      if( isset($duration[1]) || isset($splash[1]) || isset($caption[1]) ) {
        $videoData = array();
        if( isset($duration[1]) ) {
          $videoData['duration'] = intval($duration[1]);
        }

        if( isset($splash[1]) ) {
          $videoData['thumbnail'] = esc_url(html_entity_decode($splash[1]));
        }

        if( isset($caption[1]) ) {
          $caption = htmlspecialchars($caption[1]);
          $caption = str_replace( array(';','[',']'), array('\;','(',')'), ($caption) );
          $videoData['name'] = $caption;
        }
      }
    }

    $live_data = $this->get_live_data($url);

    if( is_array($live_data) ) {
      if( $live_data['Live'] ) {
        $videoData['is_live'] = true;
      }
    }

    return $videoData;
  }

  function get_live_data($url) {
    // check if live stream
    $data = array(
      'jsonrpc' => '2.0',
      'method' => 'resolve',
      'params' => array(
        'urls' => array( 'lbry://' . $this->get_video_id($url) ),
      )
    );

    $response = wp_remote_post( 'https://api.na-backend.odysee.com/api/v1/proxy?m=resolve', array(
      'body'    => json_encode($data),
      'headers' => array(
        'Content-Type' => 'text/plain; charset=utf-8'
      ),
    ));

    if( !is_wp_error($response) ) {
      $body = wp_remote_retrieve_body( $response );

      $body = json_decode( $body, true );

      if( isset($body['result']['lbry://' . $this->get_video_id($url)]['signing_channel']['claim_id']) ) {
        $claim_id = $body['result']['lbry://' . $this->get_video_id($url)]['signing_channel']['claim_id'];

        $get_url = esc_url_raw('https://api.odysee.live/livestream/is_live?channel_claim_id=' . $claim_id );

        $response = wp_remote_get($get_url);

        if( !is_wp_error($response) ) {
          $body = json_decode($response['body'], true);

          if( isset($body['data']['Live']) ) {
            return $body['data'];
          }
        }
      }
    }

    return false;
  }

  function skip_video_checker( $skip, $media ) {
    if( $this->get_video_id($media) ) {
      $skip = true;
    }

    return $skip;
  }

  /**
   * Return video id from link
   *
   * @param mixed $url
   *
   * @return string|bool if matched id then returns it, otherwise returns false
   */
  function get_video_id($url) {
    if( is_string($url) && stripos($url,'https://odysee.com') !== false ) {
      // match id without user https://odysee.com/Halloween.Kills.2021.1080p.WEBRip:f --> Halloween.Kills.2021.1080p.WEBRip:f
      // or with user https://odysee.com/@Destiny:6/trump-promises-to-take-back-everything:0 --> @Destiny:6/trump-promises-to-take-back-everything:0
      if( preg_match('~\.com/(@?.*:.$)~', $url, $matches) ) {
        return $matches[1];
      }
    }

    return false;
  }

}

global $FV_Player_Pro_Odysee;
$FV_Player_Pro_Odysee = new FV_Player_Pro_Odysee;

endif;