<?php

if ( !class_exists('FV_Player_Pro_Transcript') ) :

class FV_Player_Pro_Transcript {

  function __construct() {
    add_action( 'admin_init', array( $this, 'cron_init' ) );
    add_action( 'fv_player_pro_update_transcript_cache', array( $this, 'update_transcript_cache' ) );
    add_filter( 'cron_schedules', array( $this, 'fv_cron_schedules' ) );
    add_filter( 'fv_flowplayer_html', array( $this, 'transcript_html' ), 11, 2 );
  }
  
  function transcript_html( $html ) {
    global $fv_fp;

    $aArgs = func_get_args();

    $transcript_video_arr = array();
    $element_id = $aArgs[1]->hash;
    $transcript_html = '';
    $aVideos = false;
    $has_transcript = false;

    if ( isset($aArgs[1]->aCurArgs['transcript']) && strlen(trim($aArgs[1]->aCurArgs['transcript'])) ) { // shortcode
      $transcript_video_arr['shortcode']['transcript'] = $aArgs[1]->aCurArgs['transcript'];
      $transcript_video_arr['shortcode']['src'] = $aArgs[1]->aCurArgs['src'];
      $has_transcript  = true;
      if ( !empty($aArgs[1]->aCurArgs['original_formatting']) ) {
        $transcript_video_arr['shortcode']['transcript_original_formatting'] = $aArgs[1]->aCurArgs['original_formatting'];
      }
    } else if( method_exists($fv_fp,'current_video') && $fv_fp->current_player() && $fv_fp->current_player()->getVideos() ) { // db player
      $aVideos = $fv_fp->current_player()->getVideos(); // Get all videos for current player
      foreach($aVideos as $video) {
        $transcript_video_arr[$video->getId()]['src'] = $video->getSrc();
        foreach ($video->getMetaData() as $meta) {
          if ($meta->getMetaKey() == 'transcript') {
            $transcript_video_arr[$video->getId()]['transcript'] = $meta->getMetaValue();
            $has_transcript = true;
          }
          if ($meta->getMetaKey() == 'transcript_original_formatting') {
            $transcript_video_arr[$video->getId()]['transcript_original_formatting'] = $meta->getMetaValue();
          }
          if ($meta->getMetaKey() == 'transcript_text') {
            $transcript_video_arr[$video->getId()]['transcript_text'] = $meta->getMetaValue();
          }
        }
      }
    }

    if( !$has_transcript ) {
      return $html;
    }

    // build transcrtipt html for every video with transcript
    $first_item = true;
    foreach( $transcript_video_arr as $video_id => $meta_values ) {
      $url = $this->get_transcript_url($meta_values['transcript'], $meta_values['src']);
      $transcriptRaw = '';

      if (isset($meta_values['transcript_text']) && !empty($meta_values['transcript_text']) ) { // db video has meta value 
        $transcriptRaw = $meta_values['transcript_text']; 
      } else if ( is_numeric($video_id) ) { // db no transcript text
        $transcriptRaw = $this->get_transcript_cache($url, $video_id, $meta_values['src']);
      } else { // shortcode
        $transcriptRaw = $this->get_transcript_cache($url, false, $meta_values['src']);
      }

      if( empty($transcriptRaw) ) {
        continue; // no transcript
      }

      $data =$this->format_transcript($transcriptRaw , !empty($meta_values['transcript_original_formatting']));
      $transcriptHtml = $data['transcriptHtml'];
      $trancsriptData = $data['trancsriptData'];

      $hide = 'style="display:none"';

      if($first_item) {
        $hide = '';
      }

      if (count($trancsriptData)) {
        $transcript_html .= '<div ' . $hide . ' class="fv_fp_transcript" id="fv_fp_transcript_' . $element_id . '" data-id="'.$video_id.'" data-player="#wpfp_'.$element_id.'" data-transcript=\''.json_encode($trancsriptData).'\'>' . $transcriptHtml . '</div>';
      }
      $first_item = false;
      // TODO: error message if one or more transcripts not found
    }

    if (!empty($transcript_html)) {
      //TODO: how to name js?
      wp_enqueue_script('fvplayer-pro-mark-js', plugins_url('js/jquery.mark.min.js',__FILE__), array('jquery'), FV_Player_Pro()->version );
      
      $class = 'fv_fp_transcript_wrapper';
      
      if( !apply_filters( 'fv_player_no_transcript_dragging', false ) || !apply_filters( 'fv_player_no_transcript_sizing', false ) ) {
        $class .= ' fv_fp_transcript_wrapper_resizable';

        wp_enqueue_script('interact-js', plugins_url('js/interact.min.js',__FILE__), array('jquery'), '1.3.4' );

        $sizer = !apply_filters( 'fv_player_no_transcript_sizing', false ) ? '<div class="fv_fp_transcript_sizer"><svg xmlns="http://www.w3.org/2000/svg" width="16" height="16" viewBox="0 0 16 16"><path d="M6.7 16l9.3-9.3v-1.4l-10.7 10.7z" /><path d="M9.7 16l6.3-6.3v-1.4l-7.7 7.7z" /><path d="M12.7 16l3.3-3.3v-1.4l-4.7 4.7z" /><path d="M15.7 16l0.3-0.3v-1.4l-1.7 1.7z" /></svg></div>' : '';
      }
      
      $theme = FV_Player_Pro()->_get_option( array('pro','transcript_theme') );
      if ( !empty($aArgs[1]->aCurArgs['transcript_theme']) ) $theme = $aArgs[1]->aCurArgs['transcript_theme'];
      
      $class .= ' fv_fp_transcript_'.$theme;
      
      if ( isset($aArgs[1]->aCurArgs['transcript_hidden']) || FV_Player_Pro()->_get_option( array('pro','transcript_hidden') ) ) {
        $class .= ' fv_fp_transcript_hidden';
      }
      
      return $html . '<div class="'.$class.'">'
              . '<div class="fv_fp_transcript_head"><div class="fv_fp_transcript_head_left"><label for="fv_fp_transcript_autoscroll' . $element_id . '">Autoscroll:</label>&nbsp;<input class="fv_fp_transcript_autoscroll" id="fv_fp_transcript_autoscroll' . $element_id . '" type="checkbox" checked /></div>'
              . '<input class="fv_fp_transcript_search" id="fv_fp_transcript_search' . $element_id . '" type="text" placeholder="Search" />'
              . '<div class="fv_fp_transcript_head_right"><button type="button" class="search_result_prev" disabled></button><button type="button" class="search_result_next" disabled></button>'
              . '<button type="button" class="toggle_transcript_collapse"></button></div>'
              .'</div>'
              . $transcript_html . $sizer .'</div>';
    } else {
      return $html . '<div class="fv_fp_transcript_wrapper">Error loading transcript. Please try again in a minute. No transcript found</div>';
    }
  }

  public static function format_transcript($transcriptRaw, $originalFormatting) {
    $transcriptLines = explode("\n", $transcriptRaw);
    $trancsriptData = array();
    $tId = 0;
    $time_last = 0;
    
    $last_line = false;
    
    foreach ($transcriptLines as $line_no => $line) {
      $matches = array();
      
      // hiding cue/chapter IDs
      if ( preg_match('/^(Chapter )?\d+$/', trim($line) ) && preg_match('/^((?:[0-9]{2}:){1,2}[0-9]{2}[,.][0-9]{3}) --\> ((?:[0-9]{2}:){1,2}[0-9]{2}[,.][0-9]{3})(.*)/', $transcriptLines[$line_no+1], $matches)  ) {
        continue;
      }
      
      if (preg_match('/^((?:[0-9]{2}:){1,2}[0-9]{2}[,.][0-9]{3}) --\> ((?:[0-9]{2}:){1,2}[0-9]{2}[,.][0-9]{3})(.*)/', $line, $matches)) {
        $caption_start = FV_Player_Pro::hms_to_seconds($matches[1]);
        $caption_end = FV_Player_Pro::hms_to_seconds($matches[2]);

        if ($originalFormatting) {
          $bShouldBreak = true;
        } else {
          $bShouldBreak = ( $caption_start - $time_last > 2 ) ? true : false;  // insert paragraph break if entries are more than 5 seconds apart
        }
        
        $trancsriptData[$tId++] = array(
          'id' => $tId - 1,
          'start' => $matches[1],
          'end' => $matches[2],
          'text' =>  $time_last && $bShouldBreak ? '<br />' : '',
        );
  
        $time_last = $caption_end;
        $last_line = 'cue';

      } else if ( count($trancsriptData) ) {
        if ( $last_line == 'cue' && strlen(trim($line)) == 0 ) {
          $trancsriptData[$tId - 1]['text'].= ( $originalFormatting ? '<br />' : '<br /><br />')."\n";
          $last_line = 'blank';
        } else if ( trim($line) ) {
          $trancsriptData[$tId - 1]['text'].= ' '.$line."\n";
          $last_line = 'text';
        }
      }
    }
    
    usort( $trancsriptData, array( 'self','transcript_sort_test') );
    $transcriptHtml = "";

    foreach ($trancsriptData as $key => $val) {
      $transcriptHtml .= '<span class="fv_fp_transcript_item" data-chapter="' . ($val['id']) . '">' . $val['text'] . ' </span>';
      unset($trancsriptData[$key]['text']);
    }
    
    $data = array("trancsriptData"=>$trancsriptData, "transcriptHtml"=>$transcriptHtml);
    return $data;
  }

  public static function transcript_sort_test($a,$b) {
    return FV_Player_Pro::hms_to_seconds($a['start']) > FV_Player_Pro::hms_to_seconds($b['start']);
  }

  function get_transcript_url($transcript, $src) {
    $transcriptUrl = '';
    if ( $transcript === 'auto' && FV_Player_Pro_Vimeo()->is_vimeo($src) ) {
      // work with data cached from Vimeo API
      $objVideo = FV_Player_Pro_Vimeo()->get_vimeo($src);
        if( $objVideo && !empty($objVideo->request->text_tracks) && !empty($objVideo->request->text_tracks[0]->url) ) {
          $transcriptUrl = $objVideo->request->text_tracks[0]->url;
          if( stripos($transcriptUrl,'http://') !== 0 && stripos($transcriptUrl,'https://') !== 0 ) {
            $transcriptUrl = 'https://vimeo.com' . $transcriptUrl;
          }
        }
    } else {
      $transcriptUrl = apply_filters( 'fv_flowplayer_resource', $transcript );
    }
    return $transcriptUrl;
  }

  function get_transcript_cache($transcriptUrl, $id, $src) {
    if ( !empty($transcriptUrl) && $transcriptUrl !== 'auto' ) {
      if($id) {
        global $FV_Player_Db;
        $objVideo = new FV_Player_Db_Video( $id, array(), $FV_Player_Db );
        $text = $objVideo->getMetaValue( 'transcript_text', true );
        $last_check = $objVideo->getMetaValue( 'transcript_expire', true );

        if( empty($last_check) ) { // first time - cron didnt run yet
          $text = $this->get_transcript_http($transcriptUrl, $src);
          if( !empty($text) ) {
            $objVideo->updateMetaValue( "transcript_expire", time() + 900 );
          }
        }
        if ( empty($text) ) {
          $text = '';
          $objVideo->updateMetaValue( "transcript_expire", time() + 60 );
        }
        
        $objVideo->updateMetaValue( "transcript_text", $text);
        
        return $text;
      
      } else {
        $option_name = $this->get_option_name($transcriptUrl);
        $option = get_option($option_name, array('transcript_expire' => 0, 'text' => '', 'src' => '', 'transcript_url' => ''));
        
        if ( $option['transcript_expire'] == 0 ) { // first time
          $text = $this->get_transcript_http($transcriptUrl, $src);
          if( !empty($text) ) {
            $option['text'] = $text;
            $option['transcript_expire'] = time() + 900; 
          }
        }

        if (empty($option['text'])) {
          $option['transcript_expire'] = time() + 60; // 1min cache , if no text 
        }
        if ( (!isset($option['src']) || empty($option['src']) && $src) ) {
          $option['src'] = $src;
        }
        if ( !isset($option['transcript_url']) || empty($option['transcript_url']) ) {
          $option['transcript_url'] = $transcriptUrl;
        }
        update_option( $option_name, $option, false );
        return $option['text'];
      }
    }
    return '';
  }

  function get_transcript_http($transcriptUrl, $src) {
    $transcript_text = '';

    // handle URL without domain
    $aURL = parse_url($transcriptUrl);
    if( !isset($aURL['host']) ) {
      $aHome = parse_url(home_url());
      $transcriptUrl = home_url($transcriptUrl);
      if( isset($aHome['path']) ) {
        $transcriptUrl = str_replace( $aHome['path'], '', $transcriptUrl );
      }
    }

    $response = wp_remote_get($transcriptUrl);
      
    if ( stripos($transcriptUrl,'vimeo.com') !== false ) {
      FV_Player_Pro_Vimeo()->log_details( " getting transcript for: ".$src.", on ".$_SERVER['REQUEST_URI']."\n", $response );
    }
    if ( !is_wp_error($response) ) {
      // You might still get "<h1>Error 410 Gone</h1>" from Vimeo
      $good_transcript = !empty($response['response']['code']) && '200' == $response['response']['code'];
      if( $good_transcript ) {
        $transcript_text = $response['body'];
      }

      FV_Player_Pro_Vimeo()->log_details( " is it a good transcript? ", $good_transcript );
    }
    
    return $transcript_text; // raw transcript
  }

  function get_option_name($transcriptUrl) {
    $option_name = 'fv_player_pro_transcript_';
    if ( $query_string = strpos($transcriptUrl, "?") ) {
      $option_name .= sanitize_title( substr($transcriptUrl, 0, $query_string) );
    } else {
      $option_name .= sanitize_title($transcriptUrl);
    }
    return $option_name;
  }

  public function update_transcript_cache() {
    global $wpdb;

    // get videos with transcript metadata
    $aVideos = $wpdb->get_results( "SELECT `{$wpdb->prefix}fv_player_videos`.id as id, `{$wpdb->prefix}fv_player_videos`.src as src FROM `{$wpdb->prefix}fv_player_videos` INNER JOIN `{$wpdb->prefix}fv_player_videometa` ON `{$wpdb->prefix}fv_player_videos`.id = `{$wpdb->prefix}fv_player_videometa`.id_video WHERE `{$wpdb->prefix}fv_player_videometa`.meta_key ='transcript' " );
    if( $aVideos ) {
      foreach( $aVideos AS $objVideo ) {
        global $FV_Player_Db;

        $id = $objVideo->id;
        $src = $objVideo->src;
        $objVideo = new FV_Player_Db_Video( $id, array(), $FV_Player_Db );
        $transcript = $objVideo->getMetaValue('transcript',true);
        $last_check = $objVideo->getMetaValue('transcript_expire',true);

        if( $last_check && intval($last_check) + 900 > time() ) { // update every 15min only
          continue;
        }

        $url = $this->get_transcript_url($transcript, $src);

        $meta_new = $this->get_transcript_http($url, $src);

        if( !empty($meta_new) ) {
          $objVideo->updateMetaValue( 'transcript_expire', time() + 900 );
          $objVideo->updateMetaValue( 'transcript_text', $meta_new );
        } else {
          $objVideo->updateMetaValue( 'transcript_expire', time() + 60 );
        }
      }
    }
    
    // get data with transcript in wp_options
    $aOptions = $wpdb->get_results( "SELECT * FROM $wpdb->options WHERE option_name LIKE 'fv_player_pro_transcript_%' ");

    if($aOptions) {
      foreach( $aOptions as $option  ) {
        $values = unserialize($option->option_value);
        $last_check = $values['transcript_expire'];
        $option_name = $option->option_name;
  
        if( $last_check && intval($last_check) + 900 > time() ) { // update every 15min only
          continue;
        }
  
        $url = $values['transcript_url'];
        $src = $values['src'];
        $text = $values['text'];
  
        $option_new = $this->get_transcript_http($url, $src);
        if(!empty($option_new)) {
          update_option( $option_name, array('transcript_expire' => time() + 900, 'text' => $option_new, 'src' => $src, 'transcript_url' => $url), false );
        } else {
          update_option( $option_name, array('transcript_expire' => time() + 60, 'text' => $text, 'src' => $src, 'transcript_url' => $url), false );
        }
  
      }
    }
  }

  public function fv_cron_schedules( $schedules ) {
    if( !isset($schedules["15min"]) ) {
      $schedules["15min"] = array(
        'interval' => 15*60,
        'display' => __('Once every 15 minutes')
        );
    }
    return $schedules;
  }

  public function cron_init() {
    if ( !wp_next_scheduled( 'fv_player_pro_update_transcript_cache' ) ) {
      wp_schedule_event( time(), '15min', 'fv_player_pro_update_transcript_cache' );
    }
  }

}

global $FV_Player_Pro_Transcript;
$FV_Player_Pro_Transcript = new FV_Player_Pro_Transcript;

endif;