<?php

if( !class_exists('FV_Player_Pro_YouTube_Channel') ) :

class FV_Player_Pro_YouTube_Channel {
  
  function __construct() {
    add_filter( 'fv_flowplayer_shortcode', array( $this, 'youtube_channel' ), 10, 3 );

    // we need to try to build it later again as the src might be loaded from database
    add_filter( 'fv_flowplayer_args_pre', array( $this, 'youtube_channel' ) );

    // FV player screen
    add_filter( 'fv_flowplayer_caption_src', array( $this, 'youtube_caption' ), 10, 2 );

    add_action( 'fv_player_pro_update_youtube_cache', array( $this, 'update_youtube_cache' ) );
  }
  
  function check_youtube_cache( $attrs , $sYouTubeKey, $sListID, $sMessage, $objCache ) {
    if( !empty($attrs['channel']) ) {
      $aResponse = wp_remote_get( 'https://www.googleapis.com/youtube/v3/search?key=' . $sYouTubeKey . '&channelId='.$sListID.'&part=snippet,id&order=date&maxResults=50&type=video&order=date' );
      var_dump('https://www.googleapis.com/youtube/v3/search?key=' . $sYouTubeKey . '&channelId='.$sListID.'&part=snippet,id&order=date&maxResults=50',$aResponse);die(); // TODO: add channel support
    } else {
      $aResponse = wp_remote_get( 'https://www.googleapis.com/youtube/v3/playlistItems?part=snippet&maxResults=50&playlistId='.$sListID.'&key=' . $sYouTubeKey );
    }

    if( is_wp_error($aResponse) || (!empty($aResponse['response']['code']) && $aResponse['response']['code'] != 200) ) {
      FV_Player_Pro()->aFrontendErrors[] = '<strong>Error:</strong> YouTube API call failed.';
      return Null;
    }

    $objCacheNew = json_decode($aResponse['body']);

    if( count((array)$objCacheNew->items) ) {
      $objCache = $objCacheNew;

      $objCache->date = time();

      if( in_the_loop() && FV_Player_Pro()->is_option_enabled('vimeo_debug') ) {
        echo "<!-- FV Player Pro - $sMessage $sListID cached ".$objCache->date."-->\n";
      }

    } else {
      if( in_the_loop() && FV_Player_Pro()->is_option_enabled('vimeo_debug') ) {
        echo "<!-- FV Player Pro - $sMessage $sListID from cache ".$objCache->date." although it's old! -->\n";
      }
      $objCache->date = time() - 600;

    }

    return $objCache;
  }
  
  public function update_youtube_cache() {
    global $wpdb;

    $sYouTubeKey = FV_Player_Pro()->_get_option( array('pro','youtube_key') );

    $aYoutube = $wpdb->get_results( "SELECT id, src FROM `{$wpdb->prefix}fv_player_videos` WHERE src LIKE '%youtube%' AND ( src like '%list=%' or src like '%/channel/%' or src like '%/user/%' )" );

    if( $aYoutube ) {
      foreach( $aYoutube as $objVideo ) {
        $id = $objVideo->id;

        global $FV_Player_Db;

        $objVideo = new FV_Player_Db_Video( $id, array(), $FV_Player_Db );

        $objCache = unserialize( $objVideo->getMetaValue('playlist_data',true) );
        $playlist_last_check_duration = $objVideo->getMetaValue('playlist_last_check_date',true);

        if( $playlist_last_check_duration && intval($playlist_last_check_duration) + 900 > time() ) {
          continue;
        }

        $item = $objVideo->getSrc();
        $attrs = array();

        $type = $this->get_youtube_player_type($item);

        $attrs[$type] = true;

        preg_match( '~list=([a-zA-Z0-9-_]*+)~', $item, $aListID );

        if(!empty($aListID)) {
          $sListID = $aListID[1];
        }

        if( isset($sListID) ) {
          $sMessage = !empty($attrs['channel']) ? 'YouTube Channel' : 'YouTube Playlist';

          $objCache = $this->check_youtube_cache( $attrs, $sYouTubeKey, $sListID, $sMessage, $objCache );

          if( !empty($attrs['channel']) || !empty($attrs['playlist']) ) {
            $objVideo->updateMetaValue( "playlist_data", serialize($objCache) );
            $objVideo->updateMetaValue( "playlist_last_check_date", $objCache->date );
          }
        }

      }
    }
  }  
  
  function youtube_channel( $attrs ) {
    global $fv_fp;
    $attrs_original = $attrs;
    $objVideDBinstance = false;

    if( !isset($attrs['src']) && !isset($attrs['channel']) ) {
      return $attrs_original;
    }

    if( !empty($attrs['channel']) ) {
      $sChannelURL = trim($attrs['channel']);
      if( ( stripos($sChannelURL,'youtube.com/') === false && stripos($sChannelURL,'youtu.be/') === false ) || stripos($sChannelURL,'/channel/') === false ) {
        return $attrs_original;
      }
      
      preg_match( '~/channel/([a-zA-Z0-9-_]*+)~', $sChannelURL, $aListID );
      if( !$aListID ) {
        return $attrs_original;
      }

    } else {
      // fill-in the video arguments from database, so we enable
      // playlists parsing at this point
      if (!empty($attrs['id']) && is_numeric($attrs['id'])) {
        if (!$fv_fp->current_player() || $fv_fp->current_player()->getId() != $attrs['id']) {
          $pl = new FV_Player_Db_Player( $attrs['id'] );
        } else {
          $pl = $fv_fp->current_player();
        }
        $vids = $pl->getVideos();
        if( $vids && !empty($vids[0]) ) {
          $objVideDBinstance = $vids[0];
          $attrs = array_merge( $attrs, $objVideDBinstance->getAllDataValues() );
          unset($attrs['id']);
        }
      }

      $sPlaylistURL = trim($attrs['src']);
      if( ( stripos($sPlaylistURL,'youtube.com/') === false && stripos($sPlaylistURL,'youtu.be/') === false ) || stripos($sPlaylistURL,'list=') === false ) {
        return $attrs_original;
      }

      if( stripos($sPlaylistURL,'watch') !== false ) {
        return $attrs_original;
      }

      preg_match( '~list=([a-zA-Z0-9-_]*+)~', $sPlaylistURL, $aListID );
      if( !$aListID ) {
        return $attrs_original;
      }
        
    }
    
    $sYouTubeKey = FV_Player_Pro()->_get_option( array('pro','youtube_key') );
    if( !$sYouTubeKey  ) {
      FV_Player_Pro()->aFrontendErrors[] = '<strong>Error:</strong> Google Developer Key required.';
      return $attrs_original;
    }

    $sListID = $aListID[1];
    
    $sPrefix = !empty($attrs['channel']) ? 'fv_player_pro_youtube_channel_' : 'fv_player_pro_youtube_playlist_';
    $sMessage = !empty($attrs['channel']) ? 'YouTube Channel' : 'YouTube Playlist';

    if( is_object($objVideDBinstance) ) {
      $objCache = $objVideDBinstance->getMetaValue('playlist_data');
      if( !empty($objCache) ) {
        $objCache = unserialize($objCache[0]);
      }
    } else {
      $objCache = get_option( $sPrefix.$sListID );
    }

    $aCaption = array();
    $aPlaylist = array();

    if( $objCache && isset($objCache->date) && $objCache->date + 900 > time() ) {
      if( in_the_loop() && FV_Player_Pro()->is_option_enabled('vimeo_debug') ) {
        echo "<!-- FV Player Pro - $sMessage $sListID from cache ".$objCache->date."-->\n";
      }
    } else {
      $objCache = $this->check_youtube_cache( $attrs, $sYouTubeKey, $sListID, $sMessage, $objCache);

      if( is_null($objCache) ) {
        return $attrs_original;
      }

      if( is_object($objVideDBinstance) ) {
        $objVideDBinstance->updateMetaValue( "playlist_data", serialize($objCache) );
        $objVideDBinstance->updateMetaValue( "playlist_last_check_date", $objCache->date );
      } else {
        update_option( $sPrefix.$sListID, $objCache, false );
      }
    }
      
    $iCount = 0;
    foreach( (array) $objCache->items AS $aItem ) {
      if( empty($aItem->snippet->resourceId->videoId) ) {
        continue;
      }
      
      if( $iCount == 0 ) {
        $attrs['src'] = 'https://www.youtube.com/watch?v='.$aItem->snippet->resourceId->videoId;
        $attrs['splash'] = (isset( $aItem->snippet->thumbnails->standard ) ? $aItem->snippet->thumbnails->standard->url : $aItem->snippet->thumbnails->medium->url );
      } else {
        if( !empty($aItem->snippet->thumbnails->medium) ) {
          $aPlaylist[] = 'https://www.youtube.com/watch?v='.$aItem->snippet->resourceId->videoId.','.$aItem->snippet->thumbnails->medium->url;
        } else {
          $aPlaylist[] = 'https://www.youtube.com/watch?v='.$aItem->snippet->resourceId->videoId;
        }
      }
      $aCaption[] = str_replace( ';', ',', $aItem->snippet->title );
      $iCount++;
    }
    
    if( count($aPlaylist) ) {
      $attrs['playlist'] = implode(';',$aPlaylist);
      if( empty($attrs['caption']) ) {
        $attrs['caption'] = implode(';',$aCaption);
      }
    }

    return $attrs;
  }

  function get_youtube_player_type( $src ) {
    $type = 'single_video';
    if( stripos($src,'/channel/') || stripos($src,'/user/') ) { // channel
      $type = 'channel';
    } else if( stripos($src,'list=') ) { // playlist 
      $type = 'playlist';
    }
    return $type;
  }

  function youtube_caption( $caption, $src ) {
    if( FV_Player_Pro()->is_youtube( $src ) ) {
      $youtube = 'Youtube';
      $type = $this->get_youtube_player_type($src);

      if( strcmp( $type, 'single_video') !== 0 ) {
        $youtube .= ' ' . ucfirst($type);
      }
      $caption = $youtube . ': ' . $caption;
    }
    return $caption;
  }

}

global $FV_Player_Pro_YouTube_Channel;
$FV_Player_Pro_YouTube_Channel = new FV_Player_Pro_YouTube_Channel;

endif;
